"use strict";
var test = require("tape");
var actions_1 = require("../src/runtime/actions");
var shared_functions_1 = require("./shared_functions");
test("create a record", function (assert) {
    var expected = {
        insert: [["2", "tag", "person"], ["2", "name", "chris"],],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"chris\"]\n    ~~~\n  ");
    assert.end();
});
test("search and create a record", function (assert) {
    var expected = {
        insert: [
            ["2", "tag", "person"],
            ["2", "name", "chris"],
            ["5", "tag", "person"],
            ["5", "name", "joe"],
            ["8|2", "dude", "2"],
            ["8|5", "dude", "5"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"chris\"]\n        [#person name: \"joe\"]\n    ~~~\n\n    ~~~\n      search\n        p = [#person]\n      commit\n        [dude: p]\n    ~~~\n  ");
    assert.end();
});
test("search with constant filter", function (assert) {
    var expected = {
        insert: [
            ["2", "tag", "person"],
            ["2", "name", "chris"],
            ["5", "tag", "person"],
            ["5", "name", "joe"],
            ["9|2", "dude", "2"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"chris\"]\n        [#person name: \"joe\"]\n    ~~~\n\n    ~~~\n      search\n        p = [#person name]\n        name = \"chris\"\n      commit\n        [dude: p]\n    ~~~\n  ");
    assert.end();
});
test("search with constant attribute", function (assert) {
    var expected = {
        insert: [
            ["2", "tag", "person"],
            ["2", "name", "chris"],
            ["5", "tag", "person"],
            ["5", "name", "joe"],
            ["9|2", "dude", "2"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"chris\"]\n        [#person name: \"joe\"]\n    ~~~\n\n    ~~~\n      search\n        p = [#person name: \"chris\"]\n      commit\n        [dude: p]\n    ~~~\n  ");
    assert.end();
});
test("search with attribute having multiple values", function (assert) {
    var expected = {
        insert: [
            ["3", "tag", "person"],
            ["3", "name", "chris"],
            ["3", "name", "michael"],
            ["6", "tag", "person"],
            ["6", "name", "chris"],
            ["11|3", "dude", "3"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"chris\" name: \"michael\"]\n        [#person name: \"chris\"]\n    ~~~\n\n    ~~~\n      search\n        p = [#person name: \"chris\" name: \"michael\"]\n      commit\n        [dude: p]\n    ~~~\n  ");
    assert.end();
});
test("search with attribute having multiple values in parenthesis", function (assert) {
    var expected = {
        insert: [
            ["3", "tag", "person"],
            ["3", "name", "chris"],
            ["3", "name", "michael"],
            ["8|3", "dude", "3"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"chris\" name: \"michael\"]\n    ~~~\n\n    ~~~\n      search\n        p = [#person name: (\"chris\", \"michael\")]\n      commit\n        [dude: p]\n    ~~~\n  ");
    assert.end();
});
test("search with attribute having multiple values in parenthesis with a function", function (assert) {
    var expected = {
        insert: [
            ["3", "tag", "person"],
            ["3", "name", "chris"],
            ["3", "name", 13],
            ["9|3", "dude", "3"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"chris\" name: 13]\n    ~~~\n\n    ~~~\n      search\n        p = [#person name: (\"chris\", 4 + 9)]\n      commit\n        [dude: p]\n    ~~~\n  ");
    assert.end();
});
test.only("sub-records in a parenthesis pick up their parent as part of their identity", function (assert) {
    var expected = {
        insert: [
            ["a", "tag", "person"],
            ["a", "name", "chris"],
            ["a", "age", 20],
            ["b", "tag", "person"],
            ["b", "name", "joe"],
            ["b", "age", 20],
            ["c", "tag", "div"],
            ["c", "p", "a"],
            ["c", "children", "d"],
            ["c", "children", "e"],
            ["d", "tag", "div"],
            ["d", "text", "age"],
            ["d", "eve-auto-index", 1],
            ["e", "tag", "div"],
            ["e", "text", 20],
            ["e", "eve-auto-index", 2],
            ["f", "tag", "div"],
            ["f", "p", "b"],
            ["f", "children", "g"],
            ["f", "children", "h"],
            ["g", "tag", "div"],
            ["g", "text", "age"],
            ["g", "eve-auto-index", 1],
            ["h", "tag", "div"],
            ["h", "text", 20],
            ["h", "eve-auto-index", 2],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"chris\" age: 20]\n        [#person name: \"joe\" age: 20]\n    ~~~\n\n    ~~~\n      search\n        p = [#person name age]\n      commit\n           [#div p children: ([#div text: \"age\"], [#div text: age])]\n    ~~~\n  ");
    assert.end();
});
test("create a record with numeric attributes", function (assert) {
    var expected = {
        insert: [
            ["4", "tag", "json-array"],
            ["4", 1, "cool"],
            ["4", 2, "om nom"],
            ["4", 3, "om nom nom"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    array\n    ~~~\n      commit\n        [#json-array 1: \"cool\" 2: \"om nom\" 3: \"om nom nom\"]\n    ~~~\n  ");
    assert.end();
});
test("search a record with numeric attributes", function (assert) {
    var expected = {
        insert: [
            ["4", "tag", "json-array"],
            ["4", 1, "cool"],
            ["4", 2, "om nom"],
            ["4", 3, "om nom nom"],
            ["11", "foo", "cool - om nom - om nom nom"]
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    array\n    ~~~\n      commit\n        [#json-array 1: \"cool\" 2: \"om nom\" 3: \"om nom nom\"]\n    ~~~\n\n    ~~~\n      search\n        [#json-array 1: first, 2: second, 3: third]\n      commit\n        [| foo: \"{{first}} - {{second}} - {{third}}\"}]\n    ~~~\n  ");
    assert.end();
});
test("search with incompatible filters", function (assert) {
    var expected = {
        insert: [
            ["2", "tag", "person"],
            ["2", "name", "chris"],
            ["5", "tag", "person"],
            ["5", "name", "joe"],
        ],
        remove: [],
        errors: true,
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"chris\"]\n        [#person name: \"joe\"]\n    ~~~\n\n    ~~~\n      search\n        p = [#person name]\n        name = \"chris\"\n        name = \"joe\"\n      commit\n        [dude: p]\n    ~~~\n  ");
    assert.end();
});
test("search with unprovided variable", function (assert) {
    var expected = {
        insert: [
            ["2", "tag", "person"],
            ["2", "name", "chris"],
            ["5", "tag", "person"],
            ["5", "name", "joe"],
        ],
        remove: [],
        errors: true,
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"chris\"]\n        [#person name: \"joe\"]\n    ~~~\n\n    ~~~\n      search\n        [#person]\n      commit\n        [dude: p]\n    ~~~\n  ");
    assert.end();
});
test("search with unprovided root in an attribute access", function (assert) {
    var expected = {
        insert: [
            ["2", "tag", "person"],
            ["2", "name", "chris"],
            ["5", "tag", "person"],
            ["5", "name", "joe"],
        ],
        remove: [],
        errors: true,
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"chris\"]\n        [#person name: \"joe\"]\n    ~~~\n\n    ~~~\n      search\n        [#person]\n      commit\n        [dude: p.name]\n    ~~~\n  ");
    assert.end();
});
test("search with escaped strings", function (assert) {
    var expected = {
        insert: [
            ["3", "tag", "person"],
            ["3", "name", "chris"],
            ["3", "info", "{\"age\": 10, \"school\": \"Lincoln\"}"],
            ["7|{\"age\": 10, \"school\": \"Lincoln\"}", "info", "{\"age\": 10, \"school\": \"Lincoln\"}"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"chris\" info: \"{\\\"age\\\": 10, \\\"school\\\": \\\"Lincoln\\\"}\"]\n    ~~~\n\n    ~~~\n      search\n        [#person info]\n      commit\n        [info]\n    ~~~\n  ");
    assert.end();
});
test("search with escaped embeds", function (assert) {
    var expected = {
        insert: [
            ["2", "tag", "person"],
            ["2", "name", "chris"],
            ["7|{chris}", "info", "{chris}"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"chris\"]\n    ~~~\n\n    ~~~\n      search\n        [#person name]\n      commit\n        [info: \"\\{{{name}}\\}\"]\n    ~~~\n  ");
    assert.end();
});
test("setting an attribute", function (assert) {
    var expected = {
        insert: [
            ["2", "tag", "person"],
            ["2", "name", "chris"],
            ["2", "dude", "chris"],
            ["5", "tag", "person"],
            ["5", "name", "joe"],
            ["5", "dude", "joe"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"chris\"]\n        [#person name: \"joe\"]\n    ~~~\n\n    ~~~\n      search\n        p = [#person name]\n      commit\n        p.dude := name\n    ~~~\n  ");
    assert.end();
});
test("setting an attribute to itself", function (assert) {
    // TODO: should this really be showing name inserted twice?
    var expected = {
        insert: [
            ["2", "tag", "person"],
            ["2", "name", "chris"],
            ["2", "name", "chris"],
            ["5", "tag", "person"],
            ["5", "name", "joe"],
            ["5", "name", "joe"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"chris\"]\n        [#person name: \"joe\"]\n    ~~~\n\n    ~~~\n      search\n        p = [#person name]\n      commit\n        p.name := name\n    ~~~\n  ");
    assert.end();
});
test("setting an attribute in multiple blocks", function (assert) {
    var expected = {
        insert: [
            ["1", "tag", "person"],
            ["1", "meep", "maup"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person]\n    ~~~\n\n    stuff\n    ~~~\n      search\n        p = [#person not(meep)]\n      commit\n        p.meep := \"moop\"\n    ~~~\n\n    ~~~\n      search\n        p = [#person meep]\n      commit\n        p.meep := \"maup\"\n    ~~~\n  ");
    assert.end();
});
test("setting an attribute to multiple values", function (assert) {
    var expected = {
        insert: [
            ["2", "tag", "person"],
            ["2", "name", "chris"],
            ["2", "dude", "chris"],
            ["2", "dude", "foo"],
            ["2", "dude", 3],
            ["5", "tag", "person"],
            ["5", "name", "joe"],
            ["5", "dude", "joe"],
            ["5", "dude", "foo"],
            ["5", "dude", 3],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"chris\"]\n        [#person name: \"joe\"]\n    ~~~\n\n    ~~~\n      search\n        p = [#person name]\n      commit\n        p.dude := (name, \"foo\", 3)\n    ~~~\n  ");
    assert.end();
});
test("merging multiple values into an attribute", function (assert) {
    var expected = {
        insert: [
            ["2", "tag", "person"],
            ["2", "name", "chris"],
            ["2", "dude", "chris"],
            ["2", "dude", "foo"],
            ["2", "dude", 3],
            ["5", "tag", "person"],
            ["5", "name", "joe"],
            ["5", "dude", "joe"],
            ["5", "dude", "foo"],
            ["5", "dude", 3],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"chris\"]\n        [#person name: \"joe\"]\n    ~~~\n\n    ~~~\n      search\n        p = [#person name]\n      commit\n        p <- [dude: (name, \"foo\", 3)]\n    ~~~\n  ");
    assert.end();
});
test("merges with subobjects pick up the parent object as part of their projection", function (assert) {
    var expected = {
        insert: [
            ["a", "tag", "person"],
            ["a", "name", "chris"],
            ["b", "tag", "person"],
            ["b", "name", "chris"],
            ["a", "foo", "c"],
            ["b", "foo", "d"],
            ["c", "tag", "bar"],
            ["c", "name", "chris"],
            ["d", "tag", "bar"],
            ["d", "name", "chris"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"chris\"]\n        [#person name: \"chris\"]\n    ~~~\n\n    ~~~\n      search\n        p = [#person name]\n      commit\n        p <- [foo: [#bar name]]\n    ~~~\n  ");
    assert.end();
});
test("creating an object with multiple values for an attribute", function (assert) {
    var expected = {
        insert: [
            ["2", "tag", "person"],
            ["2", "name", "chris"],
            ["13|chris|8", "tag", "dude"],
            ["13|chris|8", "dude", "chris"],
            ["13|chris|8", "dude", "foo"],
            ["13|chris|8", "dude", 8],
            ["5", "tag", "person"],
            ["5", "name", "joe"],
            ["13|joe|8", "tag", "dude"],
            ["13|joe|8", "dude", "joe"],
            ["13|joe|8", "dude", "foo"],
            ["13|joe|8", "dude", 8],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"chris\"]\n        [#person name: \"joe\"]\n    ~~~\n\n    ~~~\n      search\n        p = [#person name]\n      commit\n        [#dude dude: (name, \"foo\", 3 + 5)]\n    ~~~\n  ");
    assert.end();
});
test("creating an object with multiple complex values for an attribute", function (assert) {
    var expected = {
        insert: [
            ["2", "tag", "person"],
            ["2", "name", "chris"],
            ["6", "tag", "foo"],
            ["8", "tag", "bar"],
            ["12", "tag", "dude"],
            ["12", "dude", "6"],
            ["12", "dude", "8"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"chris\"]\n    ~~~\n\n    ~~~\n      search\n        p = [#person name]\n      commit\n        [#dude dude: ([#foo], [#bar])]\n    ~~~\n  ");
    assert.end();
});
test("setting an attribute on an object with multiple complex values", function (assert) {
    var expected = {
        insert: [
            ["2", "tag", "person"],
            ["2", "name", "chris"],
            ["6", "tag", "foo"],
            ["8", "tag", "bar"],
            ["2", "dude", "6"],
            ["2", "dude", "8"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"chris\"]\n    ~~~\n\n    ~~~\n      search\n        p = [#person name]\n      commit\n        p.dude := ([#foo], [#bar])\n    ~~~\n  ");
    assert.end();
});
test("merging an attribute on an object with multiple complex values", function (assert) {
    var expected = {
        insert: [
            ["a", "tag", "person"],
            ["a", "name", "chris"],
            ["b", "tag", "foo"],
            ["b", "eve-auto-index", 1],
            ["c", "tag", "bar"],
            ["c", "eve-auto-index", 2],
            ["a", "dude", "b"],
            ["a", "dude", "c"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"chris\"]\n    ~~~\n\n    ~~~\n      search\n        p = [#person name]\n      commit\n        p <- [dude: [#foo] [#bar]]\n    ~~~\n  ");
    assert.end();
});
test("setting an attribute that removes a previous value", function (assert) {
    var expected = {
        insert: [
            ["3", "tag", "person"],
            ["3", "name", "chris"],
            ["3", "dude", "chris"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"chris\" dude: \"joe\"]\n    ~~~\n\n    ~~~\n      search\n        p = [#person name]\n      commit\n        p.dude := name\n    ~~~\n  ");
    assert.end();
});
test("setting an attribute on click", function (assert) {
    var expected = {
        insert: [
            ["3", "tag", "person"],
            ["3", "name", "chris"],
            ["3", "dude", "joe"],
        ],
        remove: []
    };
    var eve = shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"chris\" dude: \"joe\"]\n    ~~~\n\n    ~~~\n      search\n        [#click]\n        p = [#person name]\n      commit\n        p.dude := name\n    ~~~\n  ");
    var expected2 = {
        insert: [["3", "dude", "chris"], ["click-event", "tag", "click"]],
        remove: [["3", "dude", "joe"],]
    };
    eve.execute(expected2, [new actions_1.InsertAction("blah", "click-event", "tag", "click")]);
    assert.end();
});
test("erase a record", function (assert) {
    var expected = {
        insert: [],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"chris\" dude: \"joe\"]\n    ~~~\n\n    ~~~\n      search\n        p = [#person name]\n      commit\n        p := none\n    ~~~\n  ");
    assert.end();
});
test("erase an attribute", function (assert) {
    var expected = {
        insert: [
            ["4", "tag", "person"]
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person age: 19 age: 21 age: 30]\n    ~~~\n\n    ~~~\n      search\n        p = [#person]\n      commit\n        p.age := none\n    ~~~\n  ");
    assert.end();
});
test("sum constant", function (assert) {
    var expected = {
        insert: [
            ["a", "tag", "person"],
            ["a", "name", "joe"],
            ["b", "tag", "person"],
            ["b", "name", "chris"],
            ["c", "tag", "total"],
            ["c", "total", 2],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"joe\"]\n        [#person name: \"chris\"]\n    ~~~\n\n    ~~~\n      search\n        p = [#person]\n        total = sum[value: 1, given: p]\n      commit\n        [#total total]\n    ~~~\n  ");
    assert.end();
});
test("sum variable", function (assert) {
    var expected = {
        insert: [
            ["3", "tag", "person"],
            ["3", "name", "joe"],
            ["3", "age", 10],
            ["7", "tag", "person"],
            ["7", "name", "chris"],
            ["7", "age", 20],
            ["13|30", "tag", "total"],
            ["13|30", "total", 30],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"joe\" age: 10]\n        [#person name: \"chris\" age: 20]\n    ~~~\n\n    ~~~\n      search\n        p = [#person age]\n        total = sum[value: age, given: p]\n      commit\n        [#total total]\n    ~~~\n  ");
    assert.end();
});
test("sum variable with multiple givens", function (assert) {
    var expected = {
        insert: [
            ["3", "tag", "person"],
            ["3", "name", "joe"],
            ["3", "age", 10],
            ["7", "tag", "person"],
            ["7", "name", "chris"],
            ["7", "age", 20],
            ["13|30", "tag", "total"],
            ["13|30", "total", 30],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"joe\" age: 10]\n        [#person name: \"chris\" age: 20]\n    ~~~\n\n    ~~~\n      search\n        p = [#person age]\n        total = sum[value: age, given: (p, age)]\n      commit\n        [#total total]\n    ~~~\n  ");
    assert.end();
});
test("sum groups", function (assert) {
    var expected = {
        insert: [
            ["3", "tag", "person"],
            ["3", "name", "joe"],
            ["3", "age", 10],
            ["7", "tag", "person"],
            ["7", "name", "chris"],
            ["7", "age", 20],
            ["11", "tag", "person"],
            ["11", "name", "mike"],
            ["11", "age", 20],
            ["17|1", "tag", "total"],
            ["17|1", "total", 1],
            ["17|2", "tag", "total"],
            ["17|2", "total", 2],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"joe\" age: 10]\n        [#person name: \"chris\" age: 20]\n        [#person name: \"mike\" age: 20]\n    ~~~\n\n    ~~~\n      search\n        p = [#person age]\n        total = sum[value: 1, given: p, per: age]\n      commit\n        [#total total]\n    ~~~\n  ");
    assert.end();
});
test("sum groups with multiple pers", function (assert) {
    var expected = {
        insert: [
            ["3", "tag", "person"],
            ["3", "name", "joe"],
            ["3", "age", 10],
            ["7", "tag", "person"],
            ["7", "name", "chris"],
            ["7", "age", 20],
            ["11", "tag", "person"],
            ["11", "name", "mike"],
            ["11", "age", 20],
            ["17|1", "tag", "total"],
            ["17|1", "total", 1],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"joe\" age: 10]\n        [#person name: \"chris\" age: 20]\n        [#person name: \"mike\" age: 20]\n    ~~~\n\n    ~~~\n      search\n        p = [#person age]\n        total = sum[value: 1, given: p, per: (age, p)]\n      commit\n        [#total total]\n    ~~~\n  ");
    assert.end();
});
test("aggregate stratification", function (assert) {
    var expected = {
        insert: [
            ["2", "tag", "person"],
            ["2", "name", "joe"],
            ["5", "tag", "person"],
            ["5", "name", "chris"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"joe\"]\n        [#person name: \"chris\"]\n    ~~~\n\n    ~~~\n      search\n        p = [#person]\n        total = sum[value: 1, given: p]\n        total > 2\n      commit\n        [#total total]\n    ~~~\n  ");
    assert.end();
});
test("aggregate stratification with results", function (assert) {
    var expected = {
        insert: [
            ["2", "tag", "person"],
            ["2", "name", "joe"],
            ["5", "tag", "person"],
            ["5", "name", "chris"],
            ["11|12", "tag", "total"],
            ["11|12", "total", 12],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"joe\"]\n        [#person name: \"chris\"]\n    ~~~\n\n    ~~~\n      search\n        p = [#person]\n        total = sum[value: 1, given: p]\n        total-plus-10 = total + 10\n      commit\n        [#total total: total-plus-10]\n    ~~~\n  ");
    assert.end();
});
test("aggregate stratification result joined with an object", function (assert) {
    var expected = {
        insert: [
            ["a", "tag", "person"],
            ["a", "name", "joe"],
            ["b", "tag", "person"],
            ["b", "name", "chris"],
            ["c", "tag", "expected"],
            ["c", "total", 2],
            ["d", "tag", "success"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#expected total: 2]\n        [#person name: \"joe\"]\n        [#person name: \"chris\"]\n    ~~~\n\n    ~~~\n      search\n        expected = [#expected]\n        p = [#person]\n        expected.total = count[given: p]\n      commit\n        [#success]\n    ~~~\n  ");
    assert.end();
});
test("aggregate stratification result fails to join with an object", function (assert) {
    var expected = {
        insert: [
            ["a", "tag", "person"],
            ["a", "name", "joe"],
            ["b", "tag", "person"],
            ["b", "name", "chris"],
            ["c", "tag", "expected"],
            ["c", "total", 3],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#expected total: 3]\n        [#person name: \"joe\"]\n        [#person name: \"chris\"]\n    ~~~\n\n    ~~~\n      search\n        expected = [#expected]\n        p = [#person]\n        expected.total = count[given: p]\n      commit\n        [#success]\n    ~~~\n  ");
    assert.end();
});
test("aggregate stratification with another aggregate", function (assert) {
    var expected = {
        insert: [
            ["a", "tag", "person"],
            ["a", "name", "joe"],
            ["a", "age", 10],
            ["7", "tag", "person"],
            ["7", "name", "chris"],
            ["7", "age", 20],
            ["11", "tag", "person"],
            ["11", "name", "mike"],
            ["11", "age", 20],
            ["18|3", "tag", "total"],
            ["18|3", "total", 3],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"joe\" age: 10]\n        [#person name: \"chris\" age: 20]\n        [#person name: \"mike\" age: 20]\n    ~~~\n\n    ~~~\n      search\n        p = [#person age]\n        total = sum[value: 1, given: p, per: age]\n        count-all = sum[value: total, given: total]\n      commit\n        [#total total: count-all]\n    ~~~\n  ");
    assert.end();
});
test("unstratifiable aggregate", function (assert) {
    assert.throws(function () {
        var expected = {
            insert: [],
            remove: []
        };
        shared_functions_1.evaluate(assert, expected, "\n      people\n      ~~~\n        commit\n          [#person name: \"joe\" age: 10]\n          [#person name: \"chris\" age: 20]\n          [#person name: \"mike\" age: 20]\n      ~~~\n\n\n      ~~~\n        search\n          p = [#person age]\n          total = sum[value: 1, given: count-all, per: age]\n          count-all = sum[value: total, given: total]\n        commit\n          [#total total: count-all]\n      ~~~\n    ");
    }, "Unstratifiable aggregates should throw an error");
    assert.end();
});
test("single argument is", function (assert) {
    var expected = {
        insert: [["7|false|true", "tag", "result"], ["7|false|true", "result", false], ["7|false|true", "result2", true]],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    is test\n    ~~~\n      search\n        result = is(3 > 4)\n        result2 = is(3 < 4)\n      commit\n        [#result result result2]\n    ~~~\n  ");
    assert.end();
});
test("multiple argument is", function (assert) {
    var expected = {
        insert: [["9|true|false", "tag", "result"], ["9|true|false", "result", true], ["9|true|false", "result2", false]],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    is test\n    ~~~\n      search\n        result = is(5 > 4, 6 != 9)\n        result2 = is(5 > 4, 6 = 9)\n      commit\n        [#result result result2]\n    ~~~\n  ");
    assert.end();
});
test("block order shouldn't matter", function (assert) {
    var expected = {
        insert: [
            ["7|bye!", "tag", "result"], ["7|bye!", "result", "bye!"],
            ["7|hi!", "tag", "result"], ["7|hi!", "result", "hi!"],
            ["10", "tag", "foo"], ["10", "value", "hi!"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    is test\n    ~~~\n      search\n        result = if [#foo value] then value\n                 else \"bye!\"\n      commit\n        [#result result]\n    ~~~\n\n    add a foo\n    ~~~\n      commit\n        [#foo value: \"hi!\"]\n    ~~~\n  ");
    var expected2 = {
        insert: [
            ["10|bye!", "tag", "result"], ["10|bye!", "result", "bye!"],
            ["10|hi!", "tag", "result"], ["10|hi!", "result", "hi!"],
            ["2", "tag", "foo"], ["2", "value", "hi!"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected2, "\n    add a foo\n    ~~~\n      commit\n        [#foo value: \"hi!\"]\n    ~~~\n\n    is test\n    ~~~\n      search\n        result = if [#foo value] then value\n                 else \"bye!\"\n      commit\n        [#result result]\n    ~~~\n  ");
    assert.end();
});
test("if with variable", function (assert) {
    var expected = {
        insert: [
            ["7|bye!", "tag", "result"], ["7|bye!", "result", "bye!"],
            ["7|hi!", "tag", "result"], ["7|hi!", "result", "hi!"],
            ["10", "tag", "foo"], ["10", "value", "hi!"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    is test\n    ~~~\n      search\n        result = if [#foo value] then value\n                 else \"bye!\"\n      commit\n        [#result result]\n    ~~~\n\n    add a foo\n    ~~~\n      commit\n        [#foo value: \"hi!\"]\n    ~~~\n\n  ");
    assert.end();
});
test("else with value", function (assert) {
    var expected = {
        insert: [["6|bye!", "tag", "result"], ["6|bye!", "result", "bye!"]],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    is test\n    ~~~\n      search\n        result = if [#foo] then \"hi!\"\n                 else \"bye!\"\n      commit\n        [#result result]\n    ~~~\n  ");
    assert.end();
});
test("if with constant equality", function (assert) {
    var expected = {
        insert: [
            ["2", "tag", "foo"], ["2", "value", "hi!"],
            ["13|meh", "tag", "result"], ["13|meh", "result", "meh"]
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    add a foo\n    ~~~\n      commit\n        [#foo value: \"hi!\"]\n    ~~~\n\n    is test\n    ~~~\n      search\n        [#foo value]\n        result = if value = \"yo\" then \"cool\"\n                 else if x = \"meh\" then x\n                 else \"ok\"\n      commit\n        [#result result]\n    ~~~\n  ");
    assert.end();
});
test("if with an aggregate", function (assert) {
    var expected = {
        insert: [
            ["2", "tag", "foo"], ["2", "value", "hi!"],
            ["10|0", "tag", "result"], ["10|0", "result", 0],
            ["10|1", "tag", "result"], ["10|1", "result", 1]
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    add a foo\n    ~~~\n      commit\n        [#foo value: \"hi!\"]\n    ~~~\n\n    is test\n    ~~~\n      search\n        result = if c = count[given: [#foo]] then c\n                 else 0\n      commit\n        [#result result]\n    ~~~\n  ");
    assert.end();
});
test("if with an external equality", function (assert) {
    var expected = {
        insert: [
            ["2", "tag", "foo"], ["2", "value", "hi!"],
            ["11|1", "tag", "result"], ["11|1", "result", 1]
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    add a foo\n    ~~~\n      commit\n        [#foo value: \"hi!\"]\n    ~~~\n\n    is test\n    ~~~\n      search\n        [#foo value]\n        moof = \"hi!\"\n        result = if moof = value then 1\n                 else 0\n      commit\n        [#result result]\n    ~~~\n  ");
    assert.end();
});
test("bind adds results", function (assert) {
    var expected = {
        insert: [
            ["2", "tag", "foo"], ["2", "value", "hi!"],
            ["7|hi!", "tag", "result"], ["7|hi!", "value", "hi!"]
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    add a foo\n    ~~~\n      commit\n        [#foo value: \"hi!\"]\n    ~~~\n\n    is test\n    ~~~\n      search\n        [#foo value]\n      bind\n        [#result value]\n    ~~~\n  ");
    assert.end();
});
test("bind removes dead results", function (assert) {
    var expected = {
        insert: [
            ["2", "tag", "foo"],
            ["2", "value", "hi!"],
            ["7|hi!", "tag", "result"],
            ["7|hi!", "value", "hi!"]
        ],
        remove: []
    };
    var eve = shared_functions_1.evaluate(assert, expected, "\n    add a foo\n    ~~~\n      commit\n        [#foo value: \"hi!\"]\n    ~~~\n\n    is test\n    ~~~\n      search\n        [#foo value]\n      bind\n        [#result value]\n    ~~~\n  ");
    var expected2 = {
        insert: [],
        remove: [
            ["2", "tag", "foo"],
            ["2", "value", "hi!"],
            ["7|hi!", "tag", "result"],
            ["7|hi!", "value", "hi!"]
        ]
    };
    shared_functions_1.evaluate(assert, expected2, "\n    remove foo\n    ~~~\n    search\n      foo = [#foo]\n    commit\n      foo := none\n    ~~~\n  ", eve.session);
    assert.end();
});
test("you only search facts in the specified database", function (assert) {
    var expected = {
        insert: [
            ["2", "tag", "person"],
            ["2", "name", "chris"],
            ["5", "tag", "person"],
            ["5", "name", "joe"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"chris\"]\n        [#person name: \"joe\"]\n    ~~~\n\n    ~~~\n      search @foo\n        p = [#person]\n      commit\n        [dude: p]\n    ~~~\n  ");
    assert.end();
});
test("you can search from multiple databases", function (assert) {
    var expected = {
        insert: [
            ["2", "tag", "person"],
            ["2", "name", "chris"],
            ["5", "tag", "person"],
            ["5", "name", "joe"],
            ["8|2", "dude", "2"],
            ["8|5", "dude", "5"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"chris\"]\n      commit @foo\n        [#person name: \"joe\"]\n    ~~~\n\n    ~~~\n      search (@foo, @session)\n        p = [#person]\n      commit\n        [dude: p]\n    ~~~\n  ");
    assert.end();
});
test("writing is scoped to databases", function (assert) {
    var expected = {
        insert: [
            ["2", "tag", "person"],
            ["2", "name", "chris"],
            ["5", "tag", "person"],
            ["5", "name", "joe"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit @foo\n        [#person name: \"chris\"]\n        [#person name: \"joe\"]\n    ~~~\n\n    ~~~\n      search\n        p = [#person]\n      commit\n        [dude: p]\n    ~~~\n  ");
    assert.end();
});
test("you can write into multiple databases", function (assert) {
    var expected = {
        insert: [
            ["2", "tag", "person"],
            ["2", "name", "chris"],
            ["5", "tag", "person"],
            ["5", "name", "joe"],
            ["2", "tag", "person"],
            ["2", "name", "chris"],
            ["5", "tag", "person"],
            ["5", "name", "joe"],
            ["8|2", "dude", "2"],
            ["8|5", "dude", "5"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit (@foo, @session)\n        [#person name: \"chris\"]\n        [#person name: \"joe\"]\n    ~~~\n\n    ~~~\n      search\n        p = [#person]\n      commit\n        [dude: p]\n    ~~~\n  ");
    assert.end();
});
test("reading in a scoped write uses the search scope", function (assert) {
    var expected = {
        insert: [
            ["2", "tag", "person"],
            ["2", "name", "chris"],
            ["5", "tag", "person"],
            ["5", "name", "joe"],
            ["9|chris", "dude", "chris"],
            ["9|joe", "dude", "joe"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"chris\"]\n        [#person name: \"joe\"]\n    ~~~\n\n    ~~~\n      search\n        p = [#person]\n      commit @foo\n        [dude: p.name]\n    ~~~\n  ");
    assert.end();
});
test("reading in multiple scopes write uses the search scope", function (assert) {
    var expected = {
        insert: [
            ["2", "tag", "person"],
            ["2", "name", "chris"],
            ["5", "tag", "person"],
            ["5", "name", "joe"],
            ["8", "tag", "person"],
            ["8", "name", "woop"],
            ["12|chris", "dude", "chris"],
            ["12|joe", "dude", "joe"],
            ["12|woop", "dude", "woop"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit @blah\n        [#person name: \"chris\"]\n        [#person name: \"joe\"]\n      commit\n        [#person name: \"woop\"]\n    ~~~\n\n    ~~~\n      search (@blah, @session)\n        p = [#person]\n      commit @foo\n        [dude: p.name]\n    ~~~\n  ");
    assert.end();
});
test("scoped attribute mutators pick up the search scope", function (assert) {
    var expected = {
        insert: [
            ["6", "tag", "person"],
            ["6", "name", "chris"],
            ["6", "brother", "2|6"],
            ["2|6", "tag", "person"],
            ["2|6", "name", "ryan"],
            ["2|6", "name", "meep"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"chris\" brother: [#person name: \"ryan\"]]\n    ~~~\n\n    ~~~\n      search\n        p = [#person]\n      commit @foo\n        p.brother.name := \"meep\"\n    ~~~\n  ");
    assert.end();
});
test("multi-level attribute accesses", function (assert) {
    var expected = {
        insert: [
            ["6", "tag", "person"],
            ["6", "name", "chris"],
            ["6", "brother", "2|6"],
            ["2|6", "tag", "person"],
            ["2|6", "name", "ryan"],
            ["15|ryan", "tag", "dude"],
            ["15|ryan", "dude", "ryan"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"chris\" brother: [#person name: \"ryan\"]]\n    ~~~\n\n    ~~~\n      search\n        p = [#person]\n        p2 = [#person name: p.brother.name]\n      commit\n        [#dude dude: p2.name]\n    ~~~\n  ");
    assert.end();
});
test("split function", function (assert) {
    var expected = {
        insert: [
            ["2|foo", "dude", "foo"],
            ["2|bar", "dude", "bar"],
            ["2|baz", "dude", "baz"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    ~~~\n      search\n        token = split[text: \"foo|bar|baz\" by: \"|\"]\n      commit\n        [dude: token]\n    ~~~\n  ");
    assert.end();
});
test("split function with multiple returns", function (assert) {
    var expected = {
        insert: [
            ["3|foo|1", "dude", "foo"],
            ["3|foo|1", "index", 1],
            ["3|bar|2", "dude", "bar"],
            ["3|bar|2", "index", 2],
            ["3|baz|3", "dude", "baz"],
            ["3|baz|3", "index", 3],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    ~~~\n      search\n        (token, index) = split[text: \"foo|bar|baz\" by: \"|\"]\n      commit\n        [dude: token, index]\n    ~~~\n  ");
    assert.end();
});
test("split function with attribute returns", function (assert) {
    var expected = {
        insert: [
            ["3|foo|1", "dude", "foo"],
            ["3|foo|1", "index", 1],
            ["3|bar|2", "dude", "bar"],
            ["3|bar|2", "index", 2],
            ["3|baz|3", "dude", "baz"],
            ["3|baz|3", "index", 3],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    ~~~\n      search\n         split[text: \"foo|bar|baz\" by: \"|\", token, index]\n      commit\n        [dude: token, index]\n    ~~~\n  ");
    assert.end();
});
test("split function with fixed return", function (assert) {
    var expected = {
        insert: [
            ["4|bar", "dude", "bar"],
            ["4|bar", "index", 2],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    ~~~\n      search\n        (token, 2) = split[text: \"foo|bar|baz\" by: \"|\"]\n      commit\n        [dude: token, index: 2]\n    ~~~\n  ");
    assert.end();
});
test("split function with fixed return attribute", function (assert) {
    var expected = {
        insert: [
            ["4|bar", "dude", "bar"],
            ["4|bar", "index", 2],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    ~~~\n      search\n        split[text: \"foo|bar|baz\" by: \"|\", token, index: 2]\n      commit\n        [dude: token, index: 2]\n    ~~~\n  ");
    assert.end();
});
test("split function with fixed token", function (assert) {
    var expected = {
        insert: [
            ["4|2", "dude", "bar"],
            ["4|2", "index", 2],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    ~~~\n      search\n        split[text: \"foo|bar|baz\" by: \"|\", token: \"bar\", index]\n      commit\n        [dude: \"bar\", index]\n    ~~~\n  ");
    assert.end();
});
test("split function with both fixed", function (assert) {
    var expected = {
        insert: [
            ["5", "dude", "bar"],
            ["5", "index", 2],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    ~~~\n      search\n        split[text: \"foo|bar|baz\" by: \"|\", token: \"bar\", index: 2]\n      commit\n        [dude: \"bar\", index: 2]\n    ~~~\n  ");
    assert.end();
});
test("pipe allows you to select ", function (assert) {
    var expected = {
        insert: [
            ["2", "tag", "person"],
            ["2", "name", "chris"],
            ["5", "tag", "person"],
            ["5", "name", "joe"],
            ["10|2", "dude", "2"],
            ["10|2", "name", "chris"],
            ["10|5", "dude", "5"],
            ["10|5", "name", "joe"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"chris\"]\n        [#person name: \"joe\"]\n    ~~~\n\n    ~~~\n      search\n        p = [#person name]\n      commit\n        [dude: p | name]\n    ~~~\n  ");
    assert.end();
});
test("blank lookup errors", function (assert) {
    var expected = {
        insert: [],
        remove: [],
        errors: true
    };
    shared_functions_1.evaluate(assert, expected, "\n    ~~~\n      search\n        lookup[]\n    ~~~\n  ");
    assert.end();
});
test("lookup with bound record", function (assert) {
    var expected = {
        insert: [
            ["2", "tag", "person"],
            ["2", "name", "chris"],
            ["7", "info", "Has tag with value person"],
            ["7", "info", "Has name with value chris"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"chris\"]\n    ~~~\n\n    ~~~\n      search\n        lookup[record: [#person], attribute, value]\n      commit\n        [| info: \"Has {{attribute}} with value {{value}}\"]\n    ~~~\n  ");
    assert.end();
});
test("lookup with bound attribute", function (assert) {
    var expected = {
        insert: [
            ["2", "tag", "person"],
            ["2", "name", "chris"],
            ["6", "record", "2"],
            ["6", "value", "chris"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"chris\"]\n    ~~~\n\n    ~~~\n      search\n        lookup[record, attribute: \"name\", value]\n      commit\n        [| record value]\n    ~~~\n  ");
    assert.end();
});
test("lookup with free attribute, node and bound value", function (assert) {
    var expected = {
        insert: [
            ["2", "tag", "person"],
            ["2", "name", "chris"],
            ["7", "record", "2"],
            ["7", "attribute", "name"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"chris\"]\n    ~~~\n\n    ~~~\n      search\n        record = [#person]\n        lookup[record, attribute, value: \"chris\", node]\n      commit\n        [| record attribute]\n    ~~~\n  ");
    assert.end();
});
test("lookup on node", function (assert) {
    var expected = {
        insert: [
            ["2", "tag", "person"],
            ["2", "name", "chris"],
            ["6", "record", "2"],
            ["6", "attribute", "tag"],
            ["6", "value", "person"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"chris\"]\n    ~~~\n\n    ~~~\n      search\n        lookup[record, attribute, value, node: \"0|block|0|node|3|build\"]\n      commit\n        [| record attribute value]\n    ~~~\n  ");
    assert.end();
});
test("lookup all free", function (assert) {
    var expected = {
        insert: [
            ["2", "tag", "person"],
            ["2", "name", "chris"],
            ["6", "record", "2"],
            ["6", "attribute", "tag"],
            ["6", "value", "person"],
            ["6", "node", "0|block|0|node|3|build"],
            ["6", "attribute", "name"],
            ["6", "value", "chris"],
            ["6", "node", "0|block|0|node|5|build"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"chris\"]\n    ~~~\n\n    ~~~\n      search\n        lookup[record, attribute, value, node]\n      commit @foo\n        [| record attribute value node]\n    ~~~\n  ");
    assert.end();
});
test("lookup action", function (assert) {
    var expected = {
        insert: [
            ["2", "tag", "person"],
            ["2", "name", "chris"],
            ["2", "woo4", "yep"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"chris\"]\n    ~~~\n\n    ~~~\n      search\n        record = [#person]\n        attribute = \"woo{{1 + 3}}\"\n        value = \"yep\"\n      commit\n        lookup[record, attribute, value]\n    ~~~\n  ");
    assert.end();
});
test("lookup action without value errors", function (assert) {
    var expected = {
        insert: [
            ["2", "tag", "person"],
            ["2", "name", "chris"],
        ],
        remove: [],
        errors: true,
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"chris\"]\n    ~~~\n\n    ~~~\n      search\n        record = [#person]\n        attribute = \"woo{{1 + 3}}\"\n        value = \"yep\"\n      commit\n        lookup[record, attribute]\n    ~~~\n  ");
    assert.end();
});
test("lookup action remove", function (assert) {
    var expected = {
        insert: [
            ["2", "tag", "person"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"chris\"]\n    ~~~\n\n    ~~~\n      search\n        record = [#person]\n        attribute = \"name\"\n        value = \"chris\"\n      commit\n        lookup[record, attribute, value] := none\n    ~~~\n  ");
    assert.end();
});
test("lookup action remove free value", function (assert) {
    var expected = {
        insert: [
            ["2", "tag", "person"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"chris\"]\n    ~~~\n\n    ~~~\n      search\n        record = [#person]\n        attribute = \"name\"\n      commit\n        lookup[record, attribute] := none\n    ~~~\n  ");
    assert.end();
});
test("an identifier followed by whitespace should not be interpreted as a function", function (assert) {
    var expected = {
        insert: [
            ["2", "tag", "person"],
            ["2", "name", "chris"],
            ["2", "dude", "chris"],
            ["5", "tag", "person"],
            ["5", "name", "joe"],
            ["5", "dude", "joe"],
            ["10", "tag", "cool"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"chris\"]\n        [#person name: \"joe\"]\n    ~~~\n\n    ~~~\n      search\n        p = [#person name]\n      commit\n        p.dude := name\n        [#cool]\n    ~~~\n  ");
    assert.end();
});
test("indented code blocks are not evaled", function (assert) {
    var expected = {
        insert: [],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n\n        commit\n          [#person name: \"chris\"]\n          [#person name: \"joe\"]\n\n    ~~~\n      search\n        p = [#person name]\n      commit\n        p.dude := name\n        [#cool]\n    ~~~\n  ");
    assert.end();
});
test("single value sort", function (assert) {
    var expected = {
        insert: [
            ["2", "tag", "person"],
            ["2", "name", "a"],
            ["5", "tag", "person"],
            ["5", "name", "b"],
            ["8", "tag", "person"],
            ["8", "name", "c"],
            ["14|1 a", "dude", "1 a"],
            ["14|2 b", "dude", "2 b"],
            ["14|3 c", "dude", "3 c"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"a\"]\n        [#person name: \"b\"]\n        [#person name: \"c\"]\n    ~~~\n\n    ~~~\n      search\n        p = [#person name]\n        ix = sort[value: name]\n      commit\n        [dude: \"{{ix}} {{name}}\"]\n    ~~~\n  ");
    assert.end();
});
test("multi value sort", function (assert) {
    var expected = {
        insert: [
            ["3", "tag", "person"],
            ["3", "name", "a"],
            ["3", "age", 1],
            ["7", "tag", "person"],
            ["7", "name", "a"],
            ["7", "age", 2],
            ["11", "tag", "person"],
            ["11", "name", "b"],
            ["11", "age", 1],
            ["18|1 a 1", "dude", "1 a 1"],
            ["18|2 a 2", "dude", "2 a 2"],
            ["18|3 b 1", "dude", "3 b 1"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"a\" age: 1]\n        [#person name: \"a\" age: 2]\n        [#person name: \"b\" age: 1]\n    ~~~\n\n    ~~~\n      search\n        p = [#person name age]\n        ix = sort[value: (name, age)]\n      commit\n        [dude: \"{{ix}} {{name}} {{age}}\"]\n    ~~~\n  ");
    assert.end();
});
test("multi value sort with multiple directions", function (assert) {
    var expected = {
        insert: [
            ["3", "tag", "person"],
            ["3", "name", "a"],
            ["3", "age", 1],
            ["7", "tag", "person"],
            ["7", "name", "a"],
            ["7", "age", 2],
            ["11", "tag", "person"],
            ["11", "name", "b"],
            ["11", "age", 1],
            ["18|2 a 1", "dude", "2 a 1"],
            ["18|3 a 2", "dude", "3 a 2"],
            ["18|1 b 1", "dude", "1 b 1"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"a\" age: 1]\n        [#person name: \"a\" age: 2]\n        [#person name: \"b\" age: 1]\n    ~~~\n\n    ~~~\n      search\n        p = [#person name age]\n        ix = sort[value: (name, age), direction: (\"down\", \"up\")]\n      commit\n        [dude: \"{{ix}} {{name}} {{age}}\"]\n    ~~~\n  ");
    assert.end();
});
test("sort with group", function (assert) {
    var expected = {
        insert: [
            ["3", "tag", "person"],
            ["3", "name", "a"],
            ["3", "age", 1],
            ["7", "tag", "person"],
            ["7", "name", "a"],
            ["7", "age", 2],
            ["11", "tag", "person"],
            ["11", "name", "b"],
            ["11", "age", 1],
            ["18|1 a 1", "dude", "1 a 1"],
            ["18|2 a 2", "dude", "2 a 2"],
            ["18|1 b 1", "dude", "1 b 1"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"a\" age: 1]\n        [#person name: \"a\" age: 2]\n        [#person name: \"b\" age: 1]\n    ~~~\n\n    ~~~\n      search\n        p = [#person name age]\n        ix = sort[value: age, per: name]\n      commit\n        [dude: \"{{ix}} {{name}} {{age}}\"]\n    ~~~\n  ");
    assert.end();
});
test("if with expression-only arguments", function (assert) {
    var expected = {
        insert: [
            ["7|0", "tag", "div"],
            ["7|0", "text", 0],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    test\n    ~~~\n    search\n      foo = -1 + 1\n      text = if foo < 1 then foo else \"baz\"\n    bind @browser\n      [#div text]\n    ~~~\n  ");
    assert.end();
});
test("multiple inequalities in a row", function (assert) {
    var expected = {
        insert: [
            ["3", "tag", "person"],
            ["3", "name", "chris"],
            ["3", "age", 20],
            ["7", "tag", "person"],
            ["7", "name", "joe"],
            ["7", "age", 10],
            ["14|3", "dude", "3"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    people\n    ~~~\n      commit\n        [#person name: \"chris\" age: 20]\n        [#person name: \"joe\" age: 10]\n    ~~~\n\n    ~~~\n      search\n        p = [#person name age]\n        15 < age < 30\n      commit\n        [dude: p]\n    ~~~\n  ");
    assert.end();
});
test("range positive increment", function (assert) {
    var expected = {
        insert: [
            ["a", "dude", 1],
            ["a", "dude", 2],
            ["a", "dude", 3],
            ["a", "dude", 4],
            ["a", "dude", 5],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    ~~~\n      search\n        i = range[from: 1 to: 5]\n      commit\n        [| dude: i]\n    ~~~\n  ");
    assert.end();
});
test("range negative increment", function (assert) {
    var expected = {
        insert: [
            ["2", "dude", -1],
            ["2", "dude", -2],
            ["2", "dude", -3],
            ["2", "dude", -4],
            ["2", "dude", -5],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    ~~~\n      search\n        i = range[from: -1 to: -5 increment: -1]\n      commit\n        [| dude: i]\n    ~~~\n  ");
    assert.end();
});
test("range increment on an edge boundary", function (assert) {
    var expected = {
        insert: [
            ["2", "dude", 1],
            ["2", "dude", 4],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    ~~~\n      search\n        i = range[from: 1 to: 5 increment: 3]\n      commit\n        [| dude: i]\n    ~~~\n  ");
    assert.end();
});
test("range with a single increment", function (assert) {
    var expected = {
        insert: [
            ["2", "dude", 1],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    ~~~\n      search\n        i = range[from: 1 to: 5 increment: 10]\n      commit\n        [| dude: i]\n    ~~~\n  ");
    assert.end();
});
test("range with infinite increment", function (assert) {
    var expected = {
        insert: [],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    ~~~\n      search\n        i = range[from: -1 to: -5 increment: 1]\n      commit\n        [| dude: i]\n    ~~~\n  ");
    assert.end();
});
test("accessing the same attribute sequence natural joins instead of product joining", function (assert) {
    var expected = {
        insert: [
            ["a", "tag", "user"],
            ["a", "name", "Corey Montella"],
            ["5", "tag", "user"],
            ["5", "name", "Chris Granger"],
            ["14|2|23", "tag", "message"],
            ["14|2|23", "sender", "a"],
            ["14|2|23", "text", "Hello, Chris"],
            ["14|2|23", "eve-auto-index", 1],
            ["19|5|23", "tag", "message"],
            ["19|5|23", "sender", "5"],
            ["19|5|23", "text", "Hello there!"],
            ["19|5|23", "eve-auto-index", 2],
            ["23", "tag", "conversation"],
            ["23", "messages", "19|5|23"],
            ["23", "messages", "14|2|23"],
            ["34|23", "tag", "div"],
            ["34|23", "convos", "23"],
            ["34|23", "text", "Chris Granger - Hello there!"],
            ["34|23", "text", "Corey Montella - Hello, Chris"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    We have users:\n\n    ~~~\n    commit\n      [#user name: \"Corey Montella\"]\n      [#user name: \"Chris Granger\"]\n    ~~~\n\n    And we have conversations with messages between users:\n\n    ~~~\n    search\n      corey = [#user name: \"Corey Montella\"]\n      chris = [#user name: \"Chris Granger\"]\n\n    commit\n      [#conversation messages:\n        [#message sender: corey, text: \"Hello, Chris\"]\n        [#message sender: chris, text: \"Hello there!\"]]\n    ~~~\n\n    Now I want to display all the messages and their senders\n\n    ~~~\n    search\n      convos =  [#conversation]\n\n    bind @browser\n      [#div convos | text: \"{{convos.messages.sender.name}} - {{convos.messages.text}}\"]\n    ~~~\n  ");
    assert.end();
});
test("not with no external dependencies", function (assert) {
    var expected = {
        insert: [],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    ~~~\n    search\n      not (9 = 4 + 5)\n    commit @browser\n      [#success]\n    ~~~\n  ");
    expected = {
        insert: [
            ["3", "tag", "success"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    ~~~\n    search\n      not (2 = 4 + 5)\n    commit @browser\n      [#success]\n    ~~~\n  ");
    assert.end();
});
test("not can't provide a variable for an attribute access", function (assert) {
    var expected = {
        insert: [],
        remove: [],
        errors: true,
    };
    shared_functions_1.evaluate(assert, expected, "\n    ~~~\n    search\n      not(threads = [#zom])\n      foo = threads.foo\n    bind\n      [#foo foo]\n    ~~~\n  ");
    assert.end();
});
test("not without dependencies filters correctly", function (assert) {
    var expected = {
        insert: [[1, "tag", "foo"]],
        remove: [],
    };
    shared_functions_1.evaluate(assert, expected, "\n    add some stuff\n    ~~~\n    commit\n      [#foo]\n    ~~~\n\n    ~~~\n    search\n      not([#foo])\n    bind\n      [#bar]\n    ~~~\n  ");
    assert.end();
});
test("indirect constant equality in if", function (assert) {
    var expected = {
        insert: [
            ["a", "tag", "div"],
            ["a", "text", "1 is true"],
            ["b", "tag", "div"],
            ["b", "text", "2 is false"],
            ["c", "tag", "div"],
            ["c", "text", "3 is false"],
        ],
        remove: [],
    };
    shared_functions_1.evaluate(assert, expected, "\n    ~~~\n      search\n        one = 1\n        x = range[from: 1, to: 3]\n        value = if x = one then \"true\" else \"false\"\n\n      bind @browser\n        [#div text: \"{{x}} is {{value}}\"]\n    ~~~\n  ");
    assert.end();
});
test("constant filter in if", function (assert) {
    var expected = {
        insert: [
            ["a", "tag", "div"],
            ["a", "text", 3],
        ],
        remove: [],
    };
    shared_functions_1.evaluate(assert, expected, "\n    ~~~\n      search\n        x = 3\n        \"woohoo\" = if x < 3 then \"cool\"\n                   else if x >= 3 then \"woohoo\"\n\n      bind @browser\n        [#div text: x]\n    ~~~\n  ");
    assert.end();
});
test("nested if/not expressions correctly get their args set", function (assert) {
    var expected = {
        insert: [
            ["c", "tag", "item"],
            ["c", "idx", 0],
            ["c", "title", "title 0"],
            ["d", "tag", "item"],
            ["d", "idx", 1],
            ["a", "tag", "div"],
            ["a", "text", "0 - title 0"],
            ["b", "tag", "div"],
            ["b", "text", "1 - no title"],
        ],
        remove: [],
    };
    shared_functions_1.evaluate(assert, expected, "\n    add a foo\n    ~~~\n    search\n      item = [#item idx]\n      title = if not(item.title) then \"no title\" else item.title\n    bind @browser\n      [#div text: \"{{idx}} - {{title}}\"]\n    ~~~\n\n    is test\n    ~~~\n    commit\n      [#item idx: 0 title: \"title 0\"]\n      [#item idx: 1]\n    ~~~\n  ");
    assert.end();
});
test("interdependent aggregates are appropriately stratified", function (assert) {
    var expected = {
        insert: [
            ["a", "tag", "foo"],
            ["a", "a", 3],
            ["a", "a", 5],
            ["b", "tag", "bar"],
            ["b", "b", 1],
            ["b", "b", 2],
            ["b", "b", 3],
            ["c", "tag", "result"],
            ["c", "value", 3]
        ],
        remove: [],
    };
    shared_functions_1.evaluate(assert, expected, "\n    Add some test data\n    ~~~ eve\n    commit\n      [#foo a: (3, 5)]\n      [#bar b: (1, 2, 3)]\n    ~~~\n\n\n    ~~~ eve\n    search\n      [#foo a]\n      [#bar b]\n\n      a = count[given: b]\n      final = sum[value: a, given: a]\n\n    bind\n      [#result value: final]\n    ~~~\n\n  ");
    assert.end();
});
//# sourceMappingURL=join.js.map