"use strict";
var test = require("tape");
var shared_functions_1 = require("./shared_functions");
// Test Constants
// Would this fail on different architectures than mine? Not according to documentation Maybe we Should
// flush out that implicit assumption.
var delta = 0.00000000001;
var constants_list = [
    { expression: "pi[]", expectedValue: 3.141592653589793 },
    { expression: "e[]", expectedValue: 2.718281828459045 },
    { expression: "ln2[]", expectedValue: 0.6931471805599453 },
    { expression: "log2e[]", expectedValue: 1.4426950408889634 },
    { expression: "log10e[]", expectedValue: 0.4342944819032518 },
    { expression: "sqrt1/2[]", expectedValue: 0.7071067811865476 },
    { expression: "sqrt2[]", expectedValue: 1.4142135623730951 }
];
shared_functions_1.testSingleExpressionByList(constants_list);
// Test Power Function
var pow_list = [
    { expression: "pow[ value:2 by:3 ]", expectedValue: 8 },
    { expression: "pow[ value:9 by:1 / 2 ]", expectedValue: 3 }
];
shared_functions_1.testSingleExpressionByList(pow_list);
// Test Log Function
var log_list = [
    { expression: "log[ value: e[] ]", expectedValue: 1 },
    { expression: "log[ value: 10 base: 10 ]", expectedValue: 1 }
];
shared_functions_1.testSingleExpressionByList(log_list);
// Test Exp Function
var exp_list = [
    { expression: "exp[ value: 1 ]", expectedValue: 2.718281828459045 }
];
shared_functions_1.testSingleExpressionByList(exp_list);
// Test Trig Functions
var trig_list = [
    { expression: "sin[ radians: 1 ]", expectedValue: 0.8414709848078965 },
    { expression: "cos[ radians: 1 ]", expectedValue: 0.5403023058681398 },
    { expression: "tan[ radians: 1 ]", expectedValue: 1.5574077246549023 }
];
shared_functions_1.testSingleExpressionByList(trig_list);
test("Should be able to use the sin function with degrees and radians", function (assert) {
    var expected = {
        insert: [
            ["a", "floatTest", "1"],
            ["b", "floatTest", "0.9999996829318346"],
        ],
        remove: [],
    };
    shared_functions_1.evaluate(assert, expected, "\n    ~~~\n    search\n      y = sin[degrees: 90]\n      x = sin[radians: 3.14 / 2]\n\n    bind\n      [floatTest: y]\n      [floatTest: x]\n    ~~~\n  ");
    assert.end();
});
test("Should be able to use the cos function with degrees and radians", function (assert) {
    var expected = {
        insert: [
            ["a", "floatTest", "1"],
            ["b", "floatTest", "-0.9999987317275395"],
        ],
        remove: [],
    };
    shared_functions_1.evaluate(assert, expected, "\n    ~~~\n    search\n      y = cos[degrees: 0]\n      x = cos[radians: 3.14]\n\n    bind\n      [floatTest: y]\n      [floatTest: x]\n    ~~~\n  ");
    assert.end();
});
test("Should be able to use the tan function with degrees and radians", function (assert) {
    var expected = {
        insert: [
            ["a", "floatTest", "0.5773502691896257"],
            ["b", "floatTest", "0.5463024898437905"],
        ],
        remove: [],
    };
    shared_functions_1.evaluate(assert, expected, "\n    ~~~\n    search\n      y = tan[degrees: 30]\n      x = tan[radians: 0.5]\n\n    bind\n      [floatTest: y]\n      [floatTest: x]\n    ~~~\n  ");
    assert.end();
});
// Test inverse Trig
var atrig_list = [
    { expression: "asin[ value: 0.8414709848078965 ]", expectedValue: 1 },
    { expression: "acos[ value: 0.5403023058681398 ]", expectedValue: 0.9999999999999999 },
    { expression: "atan[ value: 1.5574077246549023 ]", expectedValue: 1 },
];
shared_functions_1.testSingleExpressionByList(atrig_list);
// Test Hyperbolic Functions
var hyp_list = [
    { expression: "sinh[ value: 1 ]", expectedValue: 1.1752011936438014 },
    { expression: "cosh[ value: 1 ]", expectedValue: 1.5430806348152437 },
    { expression: "tanh[ value: 1 ]", expectedValue: 0.7615941559557649 },
];
shared_functions_1.testSingleExpressionByList(hyp_list);
// Test Inverse Hyperbolic Functions
var ahyp_list = [
    { expression: "asinh[ value: 1.1752011936438014 ]", expectedValue: 1 },
    { expression: "acosh[ value: 1.5430806348152437 ]", expectedValue: 1 },
    { expression: "atanh[ value: 0.7615941559557649 ]", expectedValue: 0.9999999999999999 },
];
shared_functions_1.testSingleExpressionByList(ahyp_list);
test("Test range", function (assert) {
    var expected = {
        insert: [
            ["a", "result", "1"],
            ["b", "result", "2"],
            ["c", "result", "3"],
        ],
        remove: [],
    };
    shared_functions_1.evaluate(assert, expected, "\n    ~~~\n    search\n      x = range[from:1 to: 3 increment: 1 ]\n\n    bind\n      [result: x]\n    ~~~\n  ");
    assert.end();
});
test("Test nested functions", function (assert) {
    var expected = {
        insert: [
            ["a", "floatTest", "1"],
        ],
        remove: [],
    };
    shared_functions_1.evaluate(assert, expected, "\n    ~~~\n    search\n      x = sin[radians: pi[] / 2]\n\n    bind\n      [floatTest: x]\n    ~~~\n  ");
    assert.end();
});
// Test Floor Function
var floor_list = [
    { expression: "floor[ value: 1.0000000000000001 ]", expectedValue: 1 },
    { expression: "floor[ value: 1.999999999999999 ]", expectedValue: 1 },
];
shared_functions_1.testSingleExpressionByList(floor_list);
// Test Ceiling Function
var ceiling_list = [
    { expression: "ceiling[ value: 1.000000000000001 ]", expectedValue: 2 },
    { expression: "ceiling[ value: 1.999999999999999 ]", expectedValue: 2 },
];
shared_functions_1.testSingleExpressionByList(ceiling_list);
// Test ABS Function
var abs_list = [
    { expression: "abs[ value: -1 ]", expectedValue: 1 },
    { expression: "abs[ value: 1 ]", expectedValue: 1 },
];
shared_functions_1.testSingleExpressionByList(abs_list);
// Test Mod Function
var mod_list = [
    { expression: "mod[ value: 7 by: 3]", expectedValue: 1 },
    { expression: "mod[ value: 6 by: 3]", expectedValue: 0 },
];
shared_functions_1.testSingleExpressionByList(mod_list);
// Test Round Function
var round_list = [
    { expression: "round[ value: 1.49999999999999 ]", expectedValue: 1 },
    { expression: "round[ value: 1.5 ]", expectedValue: 2 }
];
shared_functions_1.testSingleExpressionByList(round_list);
// Test To Fixed Function
var toFixed_list = [
    { expression: "to-fixed[ value: 1.499 places: 2 ]", expectedValue: 1.50 },
];
shared_functions_1.testSingleExpressionByList(toFixed_list);
// Test Fix Function
var Fix_list = [
    { expression: "fix[ value: 1.99999999999999]", expectedValue: 1 },
    { expression: "fix[ value: -1.99999999999999]", expectedValue: -1 },
];
shared_functions_1.testSingleExpressionByList(Fix_list);
test("Test that string concatenation is still working after NaN change.", function (assert) {
    var expected = {
        insert: [
            ["a", "result", "Test Testy"],
        ],
        remove: [],
    };
    shared_functions_1.evaluate(assert, expected, "\n    ~~~\n    search\n      a = \"Test \"\n      b = \"Testy\"\n      x =  a + b\n\n    bind\n      [result: x]\n    ~~~\n  ");
    assert.end();
});
test("Divide by zero should return nothing.", function (assert) {
    var expected = {
        insert: [],
        remove: [],
    };
    shared_functions_1.evaluate(assert, expected, "\n    ~~~\n    search\n      x = 1 / 0\n\n    bind\n      [result: x]\n    ~~~\n  ");
    assert.end();
});
test("Divide by zero in an if statement should be detectable.", function (assert) {
    var expected = {
        insert: [
            ["a", "result", "Divide by zero"],
        ],
        remove: [],
    };
    shared_functions_1.evaluate(assert, expected, "\n    ~~~\n    search\n      a = 1\n      b = 0\n      x = if a / b then \"Ooops\"\n          else \"Divide by zero\"\n\n    bind\n      [result: x]\n    ~~~\n  ");
    assert.end();
});
test("ACosh < 1 should return nothing.", function (assert) {
    var expected = {
        insert: [],
        remove: [],
    };
    shared_functions_1.evaluate(assert, expected, "\n    ~~~\n    search\n      x = acosh[value: 0.999999999999999]\n\n    bind\n      [result: x]\n    ~~~\n  ");
    assert.end();
});
test("ATanH < -1 and > 1 should return nothing.", function (assert) {
    var expected = {
        insert: [],
        remove: [],
    };
    shared_functions_1.evaluate(assert, expected, "\n    ~~~\n    search\n      x = atanh[value: 1.000000000000001]\n\n    bind\n      [result: x]\n    ~~~\n  ");
    assert.end();
});
test("Test random seed", function (assert) {
    var expected = {
        insert: [
            ["a", "same-seed", "true"],
            ["b", "different-seed", "false"],
        ],
        remove: [],
    };
    shared_functions_1.evaluate(assert, expected, "\n    ~~~\n    search\n      r1 = random[seed:0]\n      r2 = random[seed:0]\n      r3 = random[seed:1]\n      same-seed = if r1 = r2 then \"true\" else \"false\"\n      different-seed = if r1 = r3 then \"true\" else \"false\"\n\n    bind\n      [same-seed]\n      [different-seed]\n    ~~~\n  ");
    assert.end();
});
test("Test gaussian seed", function (assert) {
    var expected = {
        insert: [
            ["a", "same-seed", "true"],
            ["b", "different-seed", "false"],
        ],
        remove: [],
    };
    shared_functions_1.evaluate(assert, expected, "\n    ~~~\n    search\n      g1 = gaussian[seed:0, \u03C3:1.0, \u00B5:0.0]\n      g2 = gaussian[seed:0, \u03C3:1.0, \u00B5:0.0]\n      g3 = gaussian[seed:1, \u03C3:1.0, \u00B5:0.0]\n      same-seed = if g1 = g2 then \"true\" else \"false\"\n      different-seed = if g1 = g3 then \"true\" else \"false\"\n\n    bind\n      [same-seed]\n      [different-seed]\n    ~~~\n  ");
    assert.end();
});
//# sourceMappingURL=math.js.map